function [cov] = TDOALocUknPropSpd_CovDNSP(senPos,srcLoc,c,varargin)
% [cov] = TDOALocUknPropSpd_CovDNSP(senPos,srcLoc,c,varargin)
%
% Y. Sun, K. C. Ho, Y. Yang, and L. Chen, "An asymptotically optimal
% estimator for source location and propagation speed by TDOA," IEEE Signal
% Process. Lett., vol. 30, pp. 1037-1041, Aug. 2023.
% 
% Calculate the theoretical covariance matrix of DNSP, given by eq. (34).
%
% Created by Y. Sun, SCU, Oct. 2023

% input:
%   senPos(m):	(Dim x M), the postions of reciveing sensors, each row is a
%               sensor position and first row is the reference sensor;
%   souLoc(m):	(Dim x 1), the postions of reciveing source;
%   c(m/s):     scalar, the propagation speed of signal;
%   varargin:
%       Qr:     TDOA measurements covariance matrix;
%       *Qs:    sensor positions covariance matrix;
% output:
%   cov:        theoretical covariance matrix of DNSP, (Dim+1 x Dim+1) matrix for
%               position and propagation speed.
%
% Created by Y. Sun, K. C. Ho   Oct. 2023
%

[N,M] = size(senPos);   % N = Dim, M = num. of sensors

r = sqrt(sum((repmat(srcLoc,1,size(senPos,2)) - senPos).^2, 1))';
td = (r(2:end)-r(1))/c;

% G1 = 2*(senPos(:,2:end)-senPos(:,1))';
G1 = 2*(senPos(:,2:end)-repmat(senPos(:,1),1,M-1))';
H1 = [2*td, td.^2];
B1 = 2*c*diag(r(2:end));
C1(:,1:N) = 2*kron((srcLoc-senPos(:,1))',ones(M-1,1));
for i = 1:M-1
    C1(i, (1:N)+i*N) = -2*(srcLoc-senPos(:,i+1))';
end

rho = (srcLoc-senPos(:,1))/r(1);
G4 = -[G1+c*H1(:,1)*rho', H1*[r(1);2*c]];

Qt = varargin{1};
if length(varargin) == 1
    W4 = inv(B1*Qt*B1');
elseif length(varargin) == 2
    Qs = varargin{2};
    W4 = inv(B1*Qt*B1' + C1*Qs*C1');
end

cov = inv(G4'*W4*G4);

end

