function [u, v] = TDOALocUknPropSpd_DNSP(senPos,td,varargin)
% [u, v] = TDOALocUknPropSpd_DNSP(senPos,td,varargin)
%
% Y. Sun, K. C. Ho, Y. Yang, and L. Chen, "An asymptotically optimal
% estimator for source location and propagation speed by TDOA," IEEE Signal
% Process. Lett., vol. 30, pp. 1037-1041, Aug. 2023.
% 
% Proposed solution:  double nullspace projection (DNSP)
%
% Input:
%   senPos(m):	(Dim x M), the postions of reciveing sensors, each row is a sensor position and
%                   first row is the reference sensor;
%   td(s):	    ((M-1) x 1), the measured TDOA;
%   varargin:
%       Qr:     TDOA measurements covariance matrix;
%       *Qs:    sensor positions covariance matrix;
% Output:
%   u(m):	    (Dim x 1), the estimation of source postion;
%   v(m/s):     scalar, the estimation of signal propagation speed;
%
% Created by Y. Sun, K. C. Ho   Oct. 2023
%

[N,M]=size(senPos);     % N = Dim, M = num. of sensors
Qt = varargin{1};
Qs = 0;
if length(varargin) == 2
    Qs = varargin{2};
end

% ver = 1;

ss = diag(senPos'*senPos);
h1 = ss(2:end) - ss(1);
G1 = 2*(senPos(:,2:end)-repmat(senPos(:,1),1,M-1))';
H1 = [2*td, td.^2];
C1 = [ones(M-1,N), kron(eye(M-1),ones(1,N))];

V1 = null(H1');
h2 = V1'*h1;
G2 = V1'*G1;

B1 = eye(M-1);

for iter = 1:2
    B2 = V1'*B1;
    C2 = V1'*C1;
    W2 = inv(B2*Qt*B2' + C2*Qs*C2');
    O = (G2'*W2*G2)\G2'*W2;
    u1 = O*h2;
    
    V2 = null(H1(:,2)');
    h3 = V2'*(h1-G1*u1);
    r1 = sqrt(sum((repmat(u1,1,M) - senPos).^2, 1))';
    G3 = V2'*H1(:,1)*r1(1);
    C3 = V2'*C1;
    B3 = V2'*B1;
    W3 = inv(B3*Qt*B3' + C3*Qs*C3');
    v1 = (G3'*W3*G3)\G3'*W3*h3;
    
    r2 = sqrt(sum((repmat(u1,1,M) - senPos).^2, 1))';
    B1 = -2*v1*diag(r2(2:end));
    C1(:,1:N) = 2*kron((u1-senPos(:,1))',ones(M-1,1));
    for i = 1:M-1
        C1(i, (1:N)+i*N) = -2*(u1-senPos(:,i+1))';
    end
end

rho = (u1-senPos(:,1))/r1(1);

h4 = h1 - G1*u1 - v1*H1*[r1(1);v1];
G4 = -[G1+v1*H1(:,1)*rho', H1*[r1(1);2*v1]];
W4 = inv(B1*Qt*B1' + C1*Qs*C1');
dd = (G4'*W4*G4)\G4'*W4*h4;

u = u1 - dd(1:N);
v = v1 - dd(N+1);

end

